@@ -3,6 +3,7 @@ use crate::anyhow::{anyhow, bail};
 use deno_core::error::AnyError;
 use deno_core::op;
 use serde::Deserialize;
+use serde_json::Value;
 use std::collections::HashSet;
 use std::sync::Mutex;
 use usvg::fontdb::Database;
@@ -88,7 +89,7 @@ struct TextInfo {
     weight: Option<String>,
     family: Option<String>,
     size: i32,
-    text: String,
+    text: Value,
 }
 
 impl TextInfo {
@@ -114,14 +115,33 @@ impl TextInfo {
         }
 
         let text_attrs_str = text_attrs.join(" ");
+        let mut escaped_text = String::new();
+
+        let text = match &self.text {
+            Value::String(s) => s.to_string(),
+            text => text.to_string(),
+        };
+
+        for char in text.chars() {
+            match char {
+                '<' => escaped_text.push_str("&lt;"),
+                '>' => escaped_text.push_str("&gt;"),
+                '"' => escaped_text.push_str("&quot;"),
+                '\'' => escaped_text.push_str("&apos;"),
+                '&' => escaped_text.push_str("&amp;"),
+                '\n' => escaped_text.push_str("&#xA;"),
+                '\r' => escaped_text.push_str("&#xD;"),
+                _ => escaped_text.push(char),
+            }
+        }
 
         format!(
             r#"
 <svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" version="1.1" width="100" height="100">
-    <text x="20" y="50" {text_attrs_str}>{text}</text>
+    <text x="20" y="50" {text_attrs_str}>{escaped_text}</text>
 </svg>"#,
             text_attrs_str = text_attrs_str,
-            text = self.text
+            escaped_text = escaped_text
         )
     }
 
@@ -143,6 +163,12 @@ pub fn op_text_width(text_info_str: String) -> Result<f64, AnyError> {
         Err(err) => bail!("Failed to deserialize text info: {}", err.to_string()),
     };
 
+    if let Some(text) = text_info.text.as_str() {
+        if text.trim().is_empty() {
+            return Ok(0.0);
+        }
+    }
+
     let svg = text_info.to_svg();
     if let Ok(width) = extract_text_width(&svg) {
         Ok(width)
