@@ -29,22 +29,18 @@ fn load_vl_spec(name: &str) -> serde_json::Value {
         .unwrap_or_else(|_| panic!("Failed to parse {:?} as JSON", spec_path))
 }
 
-fn load_expected_vg_spec(name: &str, vl_version: VlVersion, pretty: bool) -> Option<String> {
+fn load_expected_vg_spec(name: &str, vl_version: VlVersion) -> Option<serde_json::Value> {
     let root_path = Path::new(env!("CARGO_MANIFEST_DIR"));
     let spec_path = root_path
         .join("tests")
         .join("vl-specs")
         .join("expected")
         .join(&format!("{:?}", vl_version))
-        .join(if pretty {
-            format!("{}.vg.pretty.json", name)
-        } else {
-            format!("{}.vg.json", name)
-        });
+        .join(format!("{}.vg.json", name));
     if spec_path.exists() {
         let spec_str = fs::read_to_string(&spec_path)
             .unwrap_or_else(|_| panic!("Failed to read {:?}", spec_path));
-        Some(spec_str)
+        Some(serde_json::from_str(&spec_str).unwrap())
     } else {
         None
     }
@@ -77,7 +73,7 @@ fn load_expected_png(name: &str, vl_version: VlVersion) -> Vec<u8> {
 }
 
 #[rustfmt::skip]
-mod test_reference_specs {
+mod test_vegalite_to_vega {
     use crate::*;
     use futures::executor::block_on;
     use vl_convert_rs::VlConverter;
@@ -97,9 +93,6 @@ mod test_reference_specs {
 
         #[values("circle_binned", "seattle-weather", "stacked_bar_h")]
         name: &str,
-
-        #[values(false, true)]
-        pretty: bool,
     ) {
         initialize();
 
@@ -109,15 +102,16 @@ mod test_reference_specs {
         // Create Vega-Lite Converter and perform conversion
         let mut converter = VlConverter::new();
 
-        let vg_result = block_on(converter.vegalite_to_vega(vl_spec, vl_version, pretty));
+        let vg_result = block_on(converter.vegalite_to_vega(vl_spec, vl_version));
 
-        match load_expected_vg_spec(name, vl_version, pretty) {
+        match load_expected_vg_spec(name, vl_version) {
             Some(expected_vg_spec) => {
                 // Conversion is expected to succeed and match this
                 let vg_result = vg_result.expect("Vega-Lite to Vega conversion failed");
                 assert_eq!(vg_result, expected_vg_spec)
             }
             None => {
+                println!("{:?}", vg_result);
                 // Conversion is expected to fail
                 assert!(vg_result.is_err())
             }
@@ -136,7 +130,12 @@ mod test_svg {
 
     #[rstest]
     fn test(
-        #[values("circle_binned", "stacked_bar_h")]
+        #[values(
+            "circle_binned",
+            "stacked_bar_h",
+            "bar_chart_trellis_compact",
+            "line_with_log_scale"
+        )]
         name: &str,
     ) {
         initialize();
@@ -153,10 +152,8 @@ mod test_svg {
         let mut converter = VlConverter::new();
 
         // Convert to vega first
-        let vg_spec: serde_json::Value = serde_json::from_str(
-            &block_on(converter.vegalite_to_vega(vl_spec.clone(), vl_version, false)).unwrap(),
-        )
-            .unwrap();
+        let vg_spec =
+            block_on(converter.vegalite_to_vega(vl_spec.clone(), vl_version)).unwrap();
 
         let svg = block_on(converter.vega_to_svg(vg_spec)).unwrap();
         assert_eq!(svg, expected_svg);
@@ -188,7 +185,9 @@ mod test_png {
 
     #[rstest(name, scale,
         case("circle_binned", 1.0),
-        case("stacked_bar_h", 2.0)
+        case("stacked_bar_h", 2.0),
+        case("bar_chart_trellis_compact", 2.0),
+        case("line_with_log_scale", 2.0)
     )]
     fn test(
         name: &str,
@@ -208,10 +207,7 @@ mod test_png {
         let mut converter = VlConverter::new();
 
         // Convert to vega first
-        let vg_spec: serde_json::Value = serde_json::from_str(
-            &block_on(converter.vegalite_to_vega(vl_spec.clone(), vl_version, false)).unwrap(),
-        )
-            .unwrap();
+        let vg_spec = block_on(converter.vegalite_to_vega(vl_spec.clone(), vl_version)).unwrap();
 
         let png_data = block_on(converter.vega_to_png(vg_spec, Some(scale))).unwrap();
         assert_eq!(png_data, expected_png_data);
